from aiohttp import ClientSession
from flespi_sdk.modules.devices.device import Device
from flespi_sdk.modules.flespi_session import FlespiSession


class Devices(FlespiSession):
    def __init__(self, session: ClientSession, cid: int | None = None):
        super().__init__(session=session, cid=cid)

    async def get(self, id: int, fields: list[str] = []):
        async with self.session.get(
            f"/gw/devices/{id}",
            params=dict(fields=super().prepare_fields(fields)),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            device = result[0]
            return Device(
                id=device["id"],
                metadata=device["metadata"],
                fields={field: device[field] for field in fields},
                session=self.session,
                cid=self.cid,
            )

    async def list(self, selector: str = "all", fields: list[str] = []):
        async with self.session.get(
            f"/gw/devices/{selector}",
            headers=self.get_headers(),
            params=dict(fields=super().prepare_fields(fields)),
        ) as response:
            return [
                Device(
                    id=device["id"],
                    metadata=device["metadata"],
                    fields={field: device[field] for field in fields},
                    session=self.session,
                    cid=self.cid,
                )
                for device in await self.get_result(response)
            ]



from aiohttp import ClientSession

from flespi_sdk.modules.realms.realm_home import RealmHome
from flespi_sdk.modules.realms.realm_token_params import RealmTokenParams
from flespi_sdk.modules.realms.users import Users
from flespi_sdk.modules.item_with_metadata import ItemWithMetadata


class Realm(ItemWithMetadata):
    """
    Represents a realm in the Flespi system.
    """

    def __init__(
        self,
        id: int,
        metadata: dict,
        fields: dict,
        session: ClientSession,
        cid: int | None = None,
    ):
        super().__init__(
            id=id,
            item_path=f"platform/realms/{id}",
            metadata=metadata,
            fields=fields,
            session=session,
            cid=cid,
        )
        self.users = Users(
            realm_id=id,
            client_session=session,
            cid=cid,
        )

        self.home = RealmHome(realm_id=id, session=session, cid=cid)
        self.token_params = RealmTokenParams(realm_id=id, session=session, cid=cid)



import aiohttp

from flespi_sdk.modules.realms.realm import Realm
from flespi_sdk.modules.flespi_session import FlespiSession


class Realms(FlespiSession):
    def __init__(self, session: aiohttp.ClientSession, cid: int | None = None):
        super().__init__(session, cid)

    async def get(self, id: int, fields: list[str] = []):
        async with self.session.get(
            f"/platform/realms/{id}",
            params=dict(fields=super().prepare_fields(fields)),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            realm = result[0]
            return Realm(
                id=realm["id"],
                metadata=realm["metadata"],
                fields={field: realm[field] for field in fields},
                session=self.session,
                cid=self.cid,
            )

    async def list(self, selector: str = "all", fields: list[str] = []):
        async with self.session.get(
            f"/platform/realms/{selector}",
            params=dict(fields=super().prepare_fields(fields)),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return [
                Realm(
                    id=realm["id"],
                    metadata=realm["metadata"],
                    fields={field: realm[field] for field in fields},
                    session=self.session,
                    cid=self.cid,
                )
                for realm in result
            ]



from aiohttp import ClientSession
from flespi_sdk.modules.realms.users.user import User
from flespi_sdk.modules.flespi_session import FlespiSession


class Users(FlespiSession):
    """
    The RealmUsers class provides methods to manage users in a specific realm.
    It allows for the retrieval of user information and the management of user roles.
    """

    def __init__(
        self,
        realm_id: int,
        client_session: ClientSession,
        cid: int | None = None,
    ):
        """
        Initializes the RealmUsers class with a client instance.

        :param client: The client instance used to make API requests.
        """
        super().__init__(client_session, cid)
        self.realm_id = realm_id

    async def get(self, id: int, fields: list[str] = []) -> User:
        """
        Get a user by ID.
        :param user_id: The ID of the user to retrieve.
        :return: A User object.
        """
        async with self.session.get(
            f"/platform/realms/{self.realm_id}/users/{id}",
            params=dict(fields=super().prepare_fields(fields)),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            user = result[0]
            return User(
                id=user["id"],
                realm_id=self.realm_id,
                metadata=user["metadata"],
                fields={field: user[field] for field in fields},
                session=self.session,
                cid=self.cid,
            )

    async def list(self, selector: str = "all", fields: list[str] = []) -> list[User]:
        """
        List all users in the realm.
        :param selector: The selector to filter users.
        :return: A list of User objects.
        """
        async with self.session.get(
            f"/platform/realms/{self.realm_id}/users/{selector}",
            params=dict(fields=super().prepare_fields(fields)),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return [
                User(
                    id=user["id"],
                    realm_id=self.realm_id,
                    metadata=user["metadata"],
                    fields={field: user[field] for field in fields},
                    session=self.session,
                    cid=self.cid,
                )
                for user in result
            ]



from aiohttp import ClientSession
from flespi_sdk.modules.flespi_session import FlespiSession
from flespi_sdk.modules.metadata import Metadata


class ItemWithMetadata(FlespiSession):
    def __init__(
        self,
        id: int,
        item_path: str,
        session: ClientSession,
        metadata: dict | None = None,
        fields: dict = {},
        cid: int | None = None,
    ):
        super().__init__(session, cid)
        self.id = id
        self.item_path = item_path
        self.initial_fields = fields
        self.metadata = Metadata(
            item_path=item_path, metadata=metadata, session=session, cid=cid
        )

    async def get_name(self) -> str:
        """
        Get the name of the item.
        :return: The name of the item.
        """
        async with self.session.get(
            f"/{self.item_path}",
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result[0]["name"]

    def get_initial_fields(self):
        return self.initial_fields

    async def get_fields(self, fields: list[str]) -> dict:
        """
        Get fields for the current device.
        :return: Dictionary with fields.
        """
        async with self.session.get(
            self.item_path,
            params=dict(fields=fields),
            headers=self.get_headers(),
        ) as response:
            result = await self.get_result(response)
            return result[0]



from aiohttp import ClientSession
from flespi_sdk.modules.item_with_metadata import ItemWithMetadata


class Device(ItemWithMetadata):
    def __init__(
        self,
        id: int,
        metadata: dict,
        fields: dict,
        session: ClientSession,
        cid: int | None,
    ):
        super().__init__(
            id=id,
            metadata=metadata,
            fields=fields,
            item_path=f"gw/devices/{id}",
            session=session,
            cid=cid,
        )
